using WinRT;

using Microsoft.UI.Xaml;
using Microsoft.UI.Composition;
using Microsoft.UI.Composition.SystemBackdrops;

namespace TestProject
{
    /// <summary>
    /// 시스템 백드롭 윈도우
    /// </summary>
    public sealed partial class SystemBackdropsWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Enumeration
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 백드롭 타입 - BackdropType

        /// <summary>
        /// 백드롭 타입
        /// </summary>
        public enum BackdropType
        {
            /// <summary>
            /// Mica
            /// </summary>
            Mica,

            /// <summary>
            /// MicaAlt
            /// </summary>
            MicaAlt,

            /// <summary>
            /// DesktopAcrylicBase
            /// </summary>
            DesktopAcrylicBase,

            /// <summary>
            /// DesktopAcrylicThin
            /// </summary>
            DesktopAcrylicThin,

            /// <summary>
            /// DefaultColor
            /// </summary>
            DefaultColor,
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 윈도우즈 시스템 디스패처 큐 헬퍼
        /// </summary>
        private WindowsSystemDispatcherQueueHelper windowsSystemDispatcherQueueHelper;

        /// <summary>
        /// 현재 백드롭 타입
        /// </summary>
        private BackdropType currentBackdropType;

        /// <summary>
        /// 미카 컨트롤러
        /// </summary>
        private MicaController micaController;

        /// <summary>
        /// 데스크톱 아크릴 컨트롤러
        /// </summary>
        private DesktopAcrylicController desktopAcrylicController;

        /// <summary>
        /// 시스템 백드롭 구성
        /// </summary>
        private SystemBackdropConfiguration systemBackdropConfiguration;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SystemBackdropsWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public SystemBackdropsWindow()
        {
            InitializeComponent();

            ((FrameworkElement)Content).RequestedTheme = ElementTheme.Default;

            ExtendsContentIntoTitleBar = true;

            SetTitleBar(this.titleBarGrid);

            this.windowsSystemDispatcherQueueHelper = new WindowsSystemDispatcherQueueHelper();

            this.windowsSystemDispatcherQueueHelper.EnsureWindowsSystemDispatcherQueueController();

            SetBackdrop(BackdropType.Mica);

            this.changeBackdropButton.Click += changeBackdropButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 백드롭 설정하기 - SetBackdrop(backdropType)

        /// <summary>
        /// 백드롭 설정하기
        /// </summary>
        /// <param name="backdropType"></param>
        public void SetBackdrop(BackdropType backdropType)
        {
            this.currentBackdropType = BackdropType.DefaultColor;

            this.currentBackdropRun.Text = "None (default theme color)";

            this.changeStatusTextBlock.Text = "";

            if(this.micaController != null)
            {
                this.micaController.Dispose();

                this.micaController = null;
            }

            if(this.desktopAcrylicController != null)
            {
                this.desktopAcrylicController.Dispose();

                this.desktopAcrylicController = null;
            }

            Activated -= Window_Activated;
            Closed    -= Window_Closed;

            ((FrameworkElement)Content).ActualThemeChanged -= Window_ActualThemeChanged;

            this.systemBackdropConfiguration = null;

            if(backdropType == BackdropType.Mica)
            {
                if(TrySetMicaBackdrop(false))
                {
                    this.currentBackdropRun.Text = "Custom Mica";

                    this.currentBackdropType = backdropType;
                }
                else
                {
                    backdropType = BackdropType.DesktopAcrylicBase;

                    this.changeStatusTextBlock.Text += "  Mica isn't supported. Trying Acrylic.";
                }
            }

            if(backdropType == BackdropType.MicaAlt)
            {
                if(TrySetMicaBackdrop(true))
                {
                    this.currentBackdropRun.Text = "Custom MicaAlt";

                    this.currentBackdropType = backdropType;
                }
                else
                {
                    backdropType = BackdropType.DesktopAcrylicBase;

                    this.changeStatusTextBlock.Text += "  MicaAlt isn't supported. Trying Acrylic.";
                }
            }

            if(backdropType == BackdropType.DesktopAcrylicBase)
            {
                if(TrySetAcrylicBackdrop(false))
                {
                    this.currentBackdropRun.Text = "Custom Acrylic (Base)";

                    this.currentBackdropType = backdropType;
                }
                else
                {
                    this.changeStatusTextBlock.Text += "  Acrylic Base isn't supported. Switching to default color.";
                }
            }

            if(backdropType == BackdropType.DesktopAcrylicThin)
            {
                if(TrySetAcrylicBackdrop(true))
                {
                    this.currentBackdropRun.Text = "Custom Acrylic (Thin)";

                    this.currentBackdropType = backdropType;
                }
                else
                {
                    this.changeStatusTextBlock.Text += "  Acrylic Thin isn't supported. Switching to default color.";
                }
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 윈도우 활성화시 처리하기 - Window_Activated(sender, e)

        /// <summary>
        /// 윈도우 활성화시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Activated(object sender, WindowActivatedEventArgs e)
        {
            this.systemBackdropConfiguration.IsInputActive = e.WindowActivationState != WindowActivationState.Deactivated;
        }

        #endregion
        #region 윈도우 닫는 경우 처리하기 - Window_Closed(sender, e)

        /// <summary>
        /// 윈도우 닫는 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Closed(object sender, WindowEventArgs e)
        {
            if(this.micaController != null)
            {
                this.micaController.Dispose();

                this.micaController = null;
            }

            if(this.desktopAcrylicController != null)
            {
                this.desktopAcrylicController.Dispose();

                this.desktopAcrylicController = null;
            }

            Activated -= Window_Activated;

            this.systemBackdropConfiguration = null;
        }

        #endregion
        #region 윈도우 실제 테마 변경시 처리하기 - Window_ActualThemeChanged(sender, e)

        /// <summary>
        /// 윈도우 실제 테마 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_ActualThemeChanged(FrameworkElement sender, object e)
        {
            if(this.systemBackdropConfiguration != null)
            {
                SetConfigurationSourceTheme();
            }
        }

        #endregion
        #region Change Backdrop 버튼 클릭시 처리하기 - changeBackdropButton_Click(sender, e)

        /// <summary>
        /// Change Backdrop 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void changeBackdropButton_Click(object sender, RoutedEventArgs e)
        {
            BackdropType backdropType;

            switch(this.currentBackdropType)
            {
                case BackdropType.Mica               : backdropType = BackdropType.MicaAlt;            break;
                case BackdropType.MicaAlt            : backdropType = BackdropType.DesktopAcrylicBase; break;
                case BackdropType.DesktopAcrylicBase : backdropType = BackdropType.DesktopAcrylicThin; break;
                case BackdropType.DesktopAcrylicThin : backdropType = BackdropType.DefaultColor;       break;
                default                              :
                case BackdropType.DefaultColor       : backdropType = BackdropType.Mica;               break;
            }

            SetBackdrop(backdropType);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 구성 소스 테마 설정하기 - SetConfigurationSourceTheme()

        /// <summary>
        /// 구성 소스 테마 설정하기
        /// </summary>
        private void SetConfigurationSourceTheme()
        {
            switch(((FrameworkElement)this.Content).ActualTheme)
            {
                case ElementTheme.Dark    : this.systemBackdropConfiguration.Theme = SystemBackdropTheme.Dark;    break;
                case ElementTheme.Light   : this.systemBackdropConfiguration.Theme = SystemBackdropTheme.Light;   break;
                case ElementTheme.Default : this.systemBackdropConfiguration.Theme = SystemBackdropTheme.Default; break;
            }
        }

        #endregion
        #region Mica 백드롭 설정 시도하기 - TrySetMicaBackdrop(useMicaAlt)

        /// <summary>
        /// Mica 백드롭 설정 시도하기
        /// </summary>
        /// <param name="useMicaAlt">Mica 대체 사용 여부</param>
        /// <returns>처리 결과</returns>
        private bool TrySetMicaBackdrop(bool useMicaAlt)
        {
            if(MicaController.IsSupported())
            {
                this.systemBackdropConfiguration = new SystemBackdropConfiguration();

                Activated += Window_Activated;
                Closed    += Window_Closed;

                ((FrameworkElement)Content).ActualThemeChanged += Window_ActualThemeChanged;

                this.systemBackdropConfiguration.IsInputActive = true;

                SetConfigurationSourceTheme();

                this.micaController = new MicaController();

                this.micaController.Kind = useMicaAlt ? MicaKind.BaseAlt : MicaKind.Base;

                this.micaController.AddSystemBackdropTarget(this.As<ICompositionSupportsSystemBackdrop>());

                this.micaController.SetSystemBackdropConfiguration(this.systemBackdropConfiguration);

                return true;
            }

            return false;
        }

        #endregion
        #region 아크릴 백드롭 설정 시도하기 - TrySetAcrylicBackdrop(useAcrylicThin)

        /// <summary>
        /// 아크릴 백드롭 설정 시도하기
        /// </summary>
        /// <param name="useAcrylicThin">가는 아크릴 사용 여부</param>
        /// <returns>처리 결과</returns>
        bool TrySetAcrylicBackdrop(bool useAcrylicThin)
        {
            if(DesktopAcrylicController.IsSupported())
            {
                this.systemBackdropConfiguration = new SystemBackdropConfiguration();

                Activated += Window_Activated;
                Closed    += Window_Closed;

                ((FrameworkElement)Content).ActualThemeChanged += Window_ActualThemeChanged;

                this.systemBackdropConfiguration.IsInputActive = true;

                SetConfigurationSourceTheme();

                this.desktopAcrylicController = new DesktopAcrylicController();

                this.desktopAcrylicController.Kind = useAcrylicThin ? DesktopAcrylicKind.Thin : DesktopAcrylicKind.Base;

                this.desktopAcrylicController.AddSystemBackdropTarget(this.As<ICompositionSupportsSystemBackdrop>());

                this.desktopAcrylicController.SetSystemBackdropConfiguration(this.systemBackdropConfiguration);

                return true;
            }

            return false;
        }

        #endregion
    }
}